function idx = getancestors1(GO,goid,varargin)
% function GOanc = getancestors1(GO,goid,varargin)
%GETANCESTORS finds ancestors of a GO term.
%
%   GETANCESTORS(GO,ID) returns the numeric IDs of the ancestors of the
%   term ID including ID in the Gene Ontology object GO. ID is a
%   non-negative integer, it can also contain a numeric vector with a set
%   of IDs. 
%
%   GETANCESTORS(...,'HEIGHT', L) only searches upwards through L levels of
%   the Gene Ontology. L is a positive integer. Default is Inf.  
%
%   Example: 
%
%     ancestors = getancestors(GO,46680)
%     subontology = GO(ancestors)
%
%   See also GENEONT GENEONT.GENEONT.GETDESCENDANTS
%   GENEONT.GENEONT.GETMATRIX GENEONT.GENEONT.GETRELATIVES
%   GENEONT.GENEONT.SUBSREF GOANNOTREAD NUM2GOID  

%   References:
% [1]  http://www.geneontology.org/

%
%   GETANCESTORS(...,'EXCLUDE', true) excludes the original queried term
%   from the analysis. Default is false.

%   Copyright 2002-2005 The MathWorks, Inc.
%   $Revision: 1.1.10.1 $  $Date: 2005/11/11 16:18:07 $

% set defualts
exclude = false;
height = inf;

% check input arguments
if nargin > 1
    if rem(nargin,2) ~= 0
        error('Bioinfo:geneont:getancestors:IncorrectNumberOfArguments',...
            'Incorrect number of arguments to %s.',mfilename);
    end
    okargs = {'height','exclude'};
    for j=1:2:nargin-2
        pname = varargin{j};
        pval = varargin{j+1};
        if(isstruct(pval))
            error('Bioinfo:genoont:getancestors:StructParamError',...
                'parameter cannot be a struct');
        end
        k = find(strncmpi(pname, okargs,length(pname)));
        if isempty(k)
            error('Bioinfo:genoont:getancestors:UnknownParameterName',...
                'Unknown parameter name: %s.',pname);
        elseif length(k)>1
            error('Bioinfo:genoont:getancestors:AmbiguousParameterName',...
                'Ambiguous parameter name: %s.',pname);
        else
            switch(k)
                case 1  % height
                    if(pval >= 0 && isnumeric(pval))
                        height =  pval;
                    else
                        error('Bioinfo:genoont:getancestors:IncorrectHeight',...
                            '%s must be a positive integer',upper(char(okargs(k))));
                    end
                case 2  % exclude
                    if isempty(opttf(pval))
                        error('Bioinfo:genoont:getancestors:ExcludeNotLogical',...
                            '%s must be a logical value, true or false.',...
                            upper(char(okargs(k))));
                    end
                    exclude = pval;
            end
        end
    end
end
if isa(goid,'geneont.term')
    terms = get(goid,'id');
    if iscell(terms)
        terms = cell2mat(terms);
    end
else
    terms = goid(:);
end 

idx = zeros(numel(GO.gonumlist),1,'uint8');
cm = GO.relationmatrix;

orig_terms = terms;

l=1;        % path length +1
while numel(terms) && height>=0
    idx(GO.hashID(terms+1)) = l;
    terms = find(any(cm(:,terms+1),2))-1;
    height = height - 1;
    l=l+1;
end

if exclude
    idx(GO.hashID(orig_terms+1)) = 0;
end

% GOanc = GO.gonumlist(find(idx));
