function idx = getdescendants1(GO,goid,varargin)
%function GOdec = getdescendants1(GO,goid,varargin)
%GETDESCENDANTS finds descendants of a GO term.
%
%   GETDESCENDANTS(GO,ID) returns the numeric IDs of the descendants of the
%   term ID including ID in the Gene Ontology object GO. ID is a
%   non-negative integer, it can also contain a numeric vector with a set
%   of IDs. 
%
%   GETDESCENDANTS(...,'DEPTH', L) only searches downwards through L levels
%   of the Gene Ontology. L is a positive integer. Default is Inf.  
%
%   Example:
%
%     descendants = getdescendants(GO,5622)
%     subontology = GO(descendants)
%
%   See also GENEONT GENEONT.GENEONT.GETANCESTORS
%   GENEONT.GENEONT.GETMATRIX GENEONT.GENEONT.GETRELATIVES
%   GENEONT.GENEONT.SUBSREF GOANNOTREAD NUM2GOID

%   References:
% [1]  http://www.geneontology.org/

%
%   GETDESCENDANTS(...,'EXCLUDE',true) excludes the original queried term
%   from the analysis. Default is false.


%   Copyright 2002-2005 The MathWorks, Inc.
%   $Revision: 1.1.10.1 $  $Date: 2005/11/11 16:18:08 $

% set defaults 
exclude = false;
depth = inf;

% check  input arguments
if nargin > 1
    if rem(nargin,2) ~= 0
        error('Bioinfo:geneont:getdescendants:IncorrectNumberOfArguments',...
            'Incorrect number of arguments to %s.',mfilename);
    end
    okargs = {'depth','exclude'};
    for j=1:2:nargin-2
        pname = varargin{j};
        pval = varargin{j+1};
        if(isstruct(pval))
             error('Bioinfo:geneont:getdescendants:StructParamError',...
                'parameter cannot be a struct');
        end
        k = find(strncmpi(pname, okargs,length(pname)));
        if isempty(k)
            error('Bioinfo:geneont:getdescendants:UnknownParameterName',...
                'Unknown parameter name: %s.',pname);
        elseif length(k)>1
            error('Bioinfo:geneont:getdescendants:AmbiguousParameterName',...
                'Ambiguous parameter name: %s.',pname);
        else
            switch(k)
                case 1  % depth
                    if(pval >= 0 && isnumeric(pval))
                      depth =  pval;
                    else
                      error('Bioinfo:geneont:getdescendants:IncorrectHeight',...
                        '%s must be a positive integer',upper(char(okargs(k))));
                    end
                case 2  % exclude
                    if isempty(opttf(pval))
                        error('Bioinfo:geneont:getdescendants:ExcludeNotLogical',...
                            '%s must be a logical value, true or false.',...
                            upper(char(okargs(k))));
                    end
                    exclude = pval;
             end
        end
    end
end

if isa(goid,'geneont.term')
    terms = get(goid,'id');
    if iscell(terms)
        terms = cell2mat(terms);
    end
else
    terms = goid(:);
end 

idx = zeros(numel(GO.gonumlist),1,'uint8');
cm = GO.relationmatrix;

orig_terms = terms;

l=1;        % path length +1
while numel(terms) && depth>=0
    idx(GO.hashID(terms+1)) = l;
    terms = find(any(cm(terms+1,:),1))-1;
    depth = depth - 1;
    l=l+1;    
end

if exclude
    idx(GO.hashID(orig_terms+1)) = 0;
end

%GOdec = GO.gonumlist(find(idx));
